<aside class="sidebar">
    <div class="sidebar-header">
        <h2 id="logo-link" style="cursor: pointer;"><i class="fas fa-atom"></i> Deep Research</h2>
    </div>
    <nav class="sidebar-nav">
        <ul>
            <li {% if active_page == 'new-research' %}class="active"{% endif %} data-page="new-research">
                <a href="{{ url_for('research.index') }}"><i class="fas fa-search"></i> New Research</a>
            </li>
            <li {% if active_page == 'history' %}class="active"{% endif %} data-page="history">
                <a href="{{ url_for('research.history_page') }}"><i class="fas fa-history"></i> History</a>
            </li>
            <li {% if active_page == 'settings' %}class="active"{% endif %} data-page="settings">
                <a href="{{ url_for('research.settings_page') }}"><i class="fas fa-cog"></i> Settings</a>
            </li>
        </ul>
    </nav>
    <div class="sidebar-footer">
        <p>v0.2.0 | <i class="fas fa-brain"></i></p>
    </div>
</aside>
