<nav class="mobile-tab-bar">
    <ul>
        <li {% if active_page == 'new-research' %}class="active"{% endif %} data-page="new-research">
            <a href="{{ url_for('research.index') }}">
                <i class="fas fa-search"></i>
                <span>Research</span>
            </a>
        </li>
        <li {% if active_page == 'history' %}class="active"{% endif %} data-page="history">
            <a href="{{ url_for('research.history_page') }}">
                <i class="fas fa-history"></i>
                <span>History</span>
            </a>
        </li>
        <li {% if active_page == 'settings' %}class="active"{% endif %} data-page="settings">
            <a href="{{ url_for('research.settings_page') }}">
                <i class="fas fa-cog"></i>
                <span>Settings</span>
            </a>
        </li>
    </ul>
</nav> 