# src/local_deep_research/api/__init__.py
"""
API module for programmatic access to Local Deep Research functionality.
"""

from .research_functions import (
    analyze_documents,
    generate_report,
    get_available_collections,
    get_available_search_engines,
    quick_summary,
)

__all__ = [
    "quick_summary",
    "generate_report",
    "analyze_documents",
    "get_available_search_engines",
    "get_available_collections",
]
