"""
Local Deep Research - A tool for conducting deep research using AI.
"""

__author__ = "Your Name"
__description__ = "A tool for conducting deep research using AI"

from .__version__ import __version__
from .config.llm_config import get_llm
from .config.search_config import get_search
from .report_generator import get_report_generator


def get_advanced_search_system(strategy_name: str = "iterdrag"):
    """
    Get an instance of the advanced search system.

    Args:
        strategy_name: The name of the search strategy to use ("standard" or "iterdrag")

    Returns:
        AdvancedSearchSystem: An instance of the advanced search system
    """
    from .search_system import AdvancedSearchSystem

    return AdvancedSearchSystem(strategy_name=strategy_name)


__all__ = [
    "get_llm",
    "get_search",
    "get_report_generator",
    "get_advanced_search_system",
]
